MODULE TEST_lib_intp_rbf

  USE FORTUTF
  USE mo_lib_intp_rbf
  USE, INTRINSIC :: ISO_FORTRAN_ENV, ONLY: wp => real64
  USE mo_iconlib_kind, ONLY: dp

  IMPLICIT NONE

  PRIVATE

  PUBLIC :: TEST_rbf_vec_interpol_vertex_lib

  CONTAINS

  SUBROUTINE TEST_rbf_vec_interpol_vertex_lib

    IMPLICIT NONE

    INTEGER, PARAMETER :: nproma   = 6      ! inner loop/vector length
    INTEGER, PARAMETER :: nlev      = 5      ! number of vertical levels
    INTEGER, PARAMETER :: nblks_e   = 2      ! number of edge blocks (for p_e_in)
    INTEGER, PARAMETER :: nblks_v   = 2      ! number of vertex blocks (for rbf arrays and outputs)
    INTEGER, PARAMETER :: rbf_dim   = 6      ! fixed dimension for the RBF stencil

    ! Input arrays (assumed to be Fortran arrays in column-major order)
    REAL(dp), DIMENSION(nproma, nlev, nblks_e) :: p_e_in
    INTEGER, DIMENSION(rbf_dim, nproma, nblks_v) :: rbf_vec_idx_v, rbf_vec_blk_v
    REAL(dp), DIMENSION(rbf_dim, 2, nproma, nblks_v) :: rbf_vec_coeff_v

    ! Output arrays for reconstructed vector components.
    REAL(dp), DIMENSION(nproma, nlev, nblks_v) :: p_u_out, p_v_out

    ! Loop variables and tolerance for tests.
    INTEGER :: i, j, k
    REAL(dp) :: tol

    ! Set tolerance for floating point comparisons.
    tol = 1.0E-15_dp

    ! Initialize inputs:
    ! For this test, we set all values to 1.
    p_e_in = 1.0_dp
    rbf_vec_idx_v = 1
    rbf_vec_blk_v = 1
    rbf_vec_coeff_v = 1.0_dp

    ! Initialize outputs to zero.
    p_u_out = 0.0_dp
    p_v_out = 0.0_dp

    ! The following index parameters are used by get_indices_v_lib.
    ! For this simple test we assume a single block and the full inner range.
    ! (If get_indices_v_lib expects 1-based indices, these are provided as such.)
    CALL rbf_vec_interpol_vertex_lib( &
    p_e_in, rbf_vec_idx_v, rbf_vec_blk_v, rbf_vec_coeff_v, &
    p_u_out, p_v_out, &
    1, 2, &         ! i_startblk, i_endblk: one vertex block.
    1, nproma, &    ! i_startidx_in, i_endidx_in: full range.
    1, nlev, &      ! slev, elev: full vertical range.
    nproma, &       ! nproma: inner loop length.
    .false., .false.)  ! lacc and acc_async flags (not used in Kokkos version)

    ! Check the outputs.
    ! Since all coefficients and inputs are 1, each output should be the sum of 6 ones.
    DO j = 1, nlev
      DO i = 1, nproma
        IF (ABS(p_u_out(i, j, 1) - 6.0_dp) > tol) THEN
          PRINT *, "Test failed: p_u_out(", i, ",", j, ",1) =", p_u_out(i, j, 1)
          STOP 1
        END IF
        IF (ABS(p_v_out(i, j, 1) - 6.0_dp) > tol) THEN
          PRINT *, "Test failed: p_v_out(", i, ",", j, ",1) =", p_v_out(i, j, 1)
          STOP 1
        END IF
      END DO
    END DO

    PRINT *, "TEST_rbf_vec_interpol_vertex_lib passed."
  END SUBROUTINE TEST_rbf_vec_interpol_vertex_lib

END MODULE TEST_lib_intp_rbf
